"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _changeCase = require("change-case");

var _chalk = _interopRequireDefault(require("chalk"));

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var minimize = _interopRequireWildcard(require("html-minifier-terser"));

var logger = _interopRequireWildcard(require("../utils/logger"));

var _getModularRoot = _interopRequireDefault(require("../utils/getModularRoot"));

var _actionPreflightCheck = _interopRequireDefault(require("../utils/actionPreflightCheck"));

var _isModularType = require("../utils/isModularType");

var _filterDependencies = require("../utils/filterDependencies");

var _getWorkspaceInfo = _interopRequireDefault(require("../utils/getWorkspaceInfo"));

var _execAsync = _interopRequireDefault(require("../utils/execAsync"));

var _getLocation = _interopRequireDefault(require("../utils/getLocation"));

var _setupEnv = require("../utils/setupEnv");

var _createPaths = _interopRequireDefault(require("../utils/createPaths"));

var _printHostingInstructions = _interopRequireDefault(require("./printHostingInstructions"));

var _fileSizeReporter = require("./fileSizeReporter");

var _checkBrowsers = require("../utils/checkBrowsers");

var _checkRequiredFiles = _interopRequireDefault(require("../utils/checkRequiredFiles"));

var _createEsbuildBrowserslistTarget = _interopRequireDefault(require("../utils/createEsbuildBrowserslistTarget"));

var _getClientEnvironment = _interopRequireDefault(require("../esbuild-scripts/config/getClientEnvironment"));

var _api = require("../esbuild-scripts/api");

var _webpackFileSizeReporter = require("./webpackFileSizeReporter");

var _esbuildFileSizeReporter = require("./esbuildFileSizeReporter");

var _getPackageDependencies = require("../utils/getPackageDependencies");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function buildStandalone(target, type) {
  // True if there's no preference set - or the preference is for webpack.
  const useWebpack = !process.env.USE_MODULAR_WEBPACK || process.env.USE_MODULAR_WEBPACK === 'true'; // True if the preferene IS set and the preference is esbuid.

  const useEsbuild = process.env.USE_MODULAR_ESBUILD && process.env.USE_MODULAR_ESBUILD === 'true'; // If you want to use webpack then we'll always use webpack. But if you've indicated
  // you want esbuild - then we'll switch you to the new fancy world.

  const isEsbuild = !useWebpack || useEsbuild; // Setup Paths

  const modularRoot = (0, _getModularRoot.default)();
  const targetDirectory = await (0, _getLocation.default)(target);
  const targetName = (0, _changeCase.paramCase)(target);
  const paths = await (0, _createPaths.default)(target);
  const isApp = type === 'app';
  await (0, _checkBrowsers.checkBrowsers)(targetDirectory);
  let previousFileSizes;

  if (isEsbuild) {
    previousFileSizes = await (0, _esbuildFileSizeReporter.esbuildMeasureFileSizesBeforeBuild)(paths.appBuild);
  } else {
    previousFileSizes = await (0, _webpackFileSizeReporter.webpackMeasureFileSizesBeforeBuild)(paths.appBuild);
  } // Warn and crash if required files are missing


  isApp ? await (0, _checkRequiredFiles.default)([paths.appHtml, paths.appIndexJs]) : await (0, _checkRequiredFiles.default)([paths.appIndexJs]);
  logger.log('Creating an optimized production build...');
  await fs.emptyDir(paths.appBuild);

  if (isApp) {
    await fs.copy(paths.appPublic, paths.appBuild, {
      dereference: true,
      filter: file => file !== paths.appHtml,
      overwrite: true
    });
  }

  let assets;
  logger.debug('Extracting dependencies from source code...'); // Retrieve dependencies for target to inform the build process

  const {
    manifest: packageDependencies,
    resolutions: packageResolutions,
    selectiveCDNResolutions
  } = await (0, _getPackageDependencies.getPackageDependencies)(target); // Get workspace info to automatically bundle workspace dependencies

  const workspaceInfo = await (0, _getWorkspaceInfo.default)(); // Split dependencies between external and bundled

  const {
    external: externalDependencies,
    bundled: bundledDependencies
  } = (0, _filterDependencies.filterDependencies)({
    dependencies: packageDependencies,
    isApp,
    workspaceInfo
  });
  const {
    external: externalResolutions,
    bundled: bundledResolutions
  } = (0, _filterDependencies.filterDependencies)({
    dependencies: packageResolutions,
    isApp,
    workspaceInfo
  });
  logger.debug(`These are the external dependencies and their resolutions: ${JSON.stringify({
    externalDependencies,
    externalResolutions
  })}`);
  logger.debug(`These are the bundled dependencies and their resolutions: ${JSON.stringify({
    bundledDependencies,
    bundledResolutions
  })}`);
  const browserTarget = (0, _createEsbuildBrowserslistTarget.default)(targetDirectory);
  let jsEntryPoint;
  let cssEntryPoint;

  if (isEsbuild) {
    const {
      default: buildEsbuildApp
    } = await Promise.resolve().then(() => _interopRequireWildcard(require('../esbuild-scripts/build')));
    const result = await buildEsbuildApp(target, paths, externalDependencies, externalResolutions, selectiveCDNResolutions, type);
    jsEntryPoint = (0, _api.getEntryPoint)(paths, result, '.js');
    cssEntryPoint = (0, _api.getEntryPoint)(paths, result, '.css');
    assets = (0, _esbuildFileSizeReporter.createEsbuildAssets)(paths, result);
  } else {
    // create-react-app doesn't support plain module outputs yet,
    // so --preserve-modules has no effect here
    const buildScript = require.resolve('modular-scripts/react-scripts/scripts/build.js'); // TODO: this shouldn't be sync


    await (0, _execAsync.default)('node', [buildScript], {
      cwd: targetDirectory,
      log: false,
      // @ts-ignore
      env: {
        ESBUILD_TARGET_FACTORY: JSON.stringify(browserTarget),
        MODULAR_ROOT: modularRoot,
        MODULAR_PACKAGE: target,
        MODULAR_PACKAGE_NAME: targetName,
        MODULAR_IS_APP: JSON.stringify(isApp),
        MODULAR_PACKAGE_DEPS: JSON.stringify({
          externalDependencies,
          bundledDependencies
        }),
        MODULAR_PACKAGE_RESOLUTIONS: JSON.stringify({
          externalResolutions,
          bundledResolutions
        }),
        MODULAR_PACKAGE_SELECTIVE_CDN_RESOLUTIONS: JSON.stringify(selectiveCDNResolutions)
      }
    });
    const statsFilePath = path.join(paths.appBuild, 'bundle-stats.json');

    try {
      var _stats$assetsByChunkN, _stats$warnings;

      const stats = await fs.readJson(statsFilePath);
      const mainEntrypoint = stats === null || stats === void 0 ? void 0 : (_stats$assetsByChunkN = stats.assetsByChunkName) === null || _stats$assetsByChunkN === void 0 ? void 0 : _stats$assetsByChunkN.main;
      jsEntryPoint = mainEntrypoint === null || mainEntrypoint === void 0 ? void 0 : mainEntrypoint.find(entryPoint => entryPoint.endsWith('.js'));
      cssEntryPoint = mainEntrypoint === null || mainEntrypoint === void 0 ? void 0 : mainEntrypoint.find(entryPoint => entryPoint.endsWith('.css'));

      if (stats !== null && stats !== void 0 && (_stats$warnings = stats.warnings) !== null && _stats$warnings !== void 0 && _stats$warnings.length) {
        logger.log(_chalk.default.yellow('Compiled with warnings.\n'));
        logger.log(stats.warnings.join('\n\n'));
        logger.log('\nSearch for the ' + _chalk.default.underline(_chalk.default.yellow('keywords')) + ' to learn more about each warning.');
      } else {
        logger.log(_chalk.default.green('Compiled successfully.\n'));
      }

      assets = (0, _webpackFileSizeReporter.createWebpackAssets)(paths, stats);
    } finally {
      await fs.remove(statsFilePath);
    }
  } // If view, write the synthetic index.html and create a trampoline file pointing to the main entrypoint
  // This is for both esbuild and webpack so it lives here. If app, instead, the public/index.html file is generated specifical in different ways.


  if (!isApp) {
    if (!jsEntryPoint) {
      throw new Error("Can't find main entrypoint after building");
    } // Create synthetic index


    const env = (0, _getClientEnvironment.default)(paths.publicUrlOrPath.slice(0, -1));
    const html = (0, _api.createSyntheticIndex)({
      cssEntryPoint,
      replacements: env.raw
    });
    await fs.writeFile(path.join(paths.appBuild, 'index.html'), await minimize.minify(html, {
      html5: true,
      collapseBooleanAttributes: true,
      collapseWhitespace: true,
      collapseInlineTagWhitespace: true,
      decodeEntities: true,
      minifyCSS: true,
      minifyJS: true,
      removeAttributeQuotes: false,
      removeComments: true,
      removeTagWhitespace: true
    })); // Create and write trampoline file

    const trampolineBuildResult = await (0, _api.createViewTrampoline)(path.basename(jsEntryPoint), paths.appSrc, externalDependencies, externalResolutions, selectiveCDNResolutions, browserTarget);
    const trampolinePath = `${paths.appBuild}/static/js/_trampoline.js`;
    await fs.writeFile(trampolinePath, trampolineBuildResult.outputFiles[0].contents);
  } // Add dependencies from source and bundled dependencies to target package.json


  const targetPackageJson = await fs.readJSON(path.join(targetDirectory, 'package.json'));
  targetPackageJson.dependencies = packageDependencies;
  targetPackageJson.bundledDependencies = Object.keys(bundledDependencies); // Copy selected fields of package.json over

  await fs.writeJSON(path.join(paths.appBuild, 'package.json'), {
    name: targetPackageJson.name,
    version: targetPackageJson.version,
    license: targetPackageJson.license,
    modular: targetPackageJson.modular,
    dependencies: targetPackageJson.dependencies,
    bundledDependencies: targetPackageJson.bundledDependencies,
    module: jsEntryPoint ? paths.publicUrlOrPath + jsEntryPoint : undefined,
    style: cssEntryPoint ? paths.publicUrlOrPath + cssEntryPoint : undefined
  }, {
    spaces: 2
  });
  (0, _fileSizeReporter.printFileSizesAfterBuild)(assets, previousFileSizes);
  (0, _printHostingInstructions.default)(target, paths.publicUrlOrPath, paths.publicUrlOrPath, paths.appBuild);
}

async function build(target, preserveModules = true, includePrivate = false) {
  const targetDirectory = await (0, _getLocation.default)(target);
  await (0, _setupEnv.setupEnvForDirectory)(targetDirectory);
  const targetType = (0, _isModularType.getModularType)(targetDirectory);

  if (targetType === 'app' || targetType === 'esm-view') {
    await buildStandalone(target, targetType);
  } else {
    const {
      buildPackage
    } = await Promise.resolve().then(() => _interopRequireWildcard(require('./buildPackage'))); // ^ we do a dynamic import here to defer the module's initial side effects
    // till when it's actually needed (i.e. now)

    await buildPackage(target, preserveModules, includePrivate);
  }
}

var _default = (0, _actionPreflightCheck.default)(build);

exports.default = _default;
//# sourceMappingURL=index.js.map