"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.check = check;

var path = _interopRequireWildcard(require("path"));

var _getModularRoot = _interopRequireDefault(require("../utils/getModularRoot"));

var _getWorkspaceInfo = _interopRequireDefault(require("../utils/getWorkspaceInfo"));

var _isModularType = require("../utils/isModularType");

var logger = _interopRequireWildcard(require("../utils/logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function check(target) {
  let valid = true;
  const modularRoot = (0, _getModularRoot.default)(); // ensure that workspaces are setup correctly with yarn
  // init is a special case where we don't already need to be in a modular repository
  // in this case there's no use checking the workspaces yet because we're setting
  // up a new folder

  const workspace = await (0, _getWorkspaceInfo.default)(target);
  /**
   * Validate the structure of the workspace to ensure there's no mismatched dependencies and that
   * all the workspaces are valid modular package types.
   */

  for (const [packageName, packageInfo] of Object.entries(workspace)) {
    if (packageInfo.mismatchedWorkspaceDependencies.length) {
      logger.error(`${packageName} has mismatchedWorkspaceDependencies ${packageInfo.mismatchedWorkspaceDependencies.join(', ')}`);
      valid = false;
    }

    if (!packageInfo.location.startsWith('packages')) {
      logger.error(`${packageName} is not located within the "/packages" directory in the repository, instead found "${packageInfo.location}"`);
      valid = false;
    }

    if (!(0, _isModularType.isValidModularType)(path.join(modularRoot, packageInfo.location))) {
      logger.error(`${packageName} at ${packageInfo.location} is not a valid modular type - Found ${(0, _isModularType.getModularType)(packageInfo.location)}`);
      valid = false;
    }

    if (packageInfo.type === 'app' && packageInfo.public) {
      logger.error(`${packageName} is marked as "public" - Modular apps should be marked as private.`);
      valid = false;
    }

    logger.debug(`${packageName} is valid.`);
  }

  return valid;
}
//# sourceMappingURL=verifyWorkspaceDependencies.js.map