"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
exports.port = port;

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var _execa = _interopRequireDefault(require("execa"));

var _rimraf = _interopRequireDefault(require("rimraf"));

var _chalk = _interopRequireDefault(require("chalk"));

var _semver = _interopRequireDefault(require("semver"));

var _changeCase = require("change-case");

var logger = _interopRequireWildcard(require("./utils/logger"));

var _getModularRoot = _interopRequireDefault(require("./utils/getModularRoot"));

var _getWorkspaceInfo = _interopRequireDefault(require("./utils/getWorkspaceInfo"));

var _actionPreflightCheck = _interopRequireDefault(require("./utils/actionPreflightCheck"));

var _gitActions = require("./utils/gitActions");

var _check = require("./check");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

process.on('SIGINT', () => {
  (0, _gitActions.stashChanges)();
  process.exit(1);
});

async function port(relativePath) {
  const modularRoot = (0, _getModularRoot.default)();

  if (!(0, _gitActions.cleanGit)(modularRoot)) {
    throw new Error('You have unsaved changes. Please save or stash them before we attempt to port this react app to your modular project.');
  }

  const targetRoot = path.resolve(modularRoot, relativePath);

  if (!fs.existsSync(path.join(targetRoot, 'package.json'))) {
    throw new Error("Couldn't find a `package.json` in your react-app directory. Unable to proceed.");
  }

  try {
    const targetedAppPackageJson = await fs.readJSON(path.join(targetRoot, 'package.json'));
    const targetedAppName = targetedAppPackageJson.name;
    logger.log(`Porting ${targetedAppName} over into packages as a modular app...`); // Create a modular app package to funnel targeted app into

    const packageTypePath = path.join(__dirname, '../types', 'app');
    const newPackagePath = path.join(modularRoot, 'packages', (0, _changeCase.paramCase)(targetedAppName));
    fs.mkdirpSync(newPackagePath); // Copy the targeted folders to the modular app

    const srcFolders = ['src', 'public'];
    srcFolders.forEach(dir => {
      if (fs.existsSync(path.join(targetRoot, dir))) {
        fs.copySync(path.join(targetRoot, dir), path.join(newPackagePath, dir), {
          overwrite: true
        });
      } else {
        fs.copySync(path.join(packageTypePath, dir), path.join(newPackagePath, dir), {
          overwrite: true
        });
      }
    });
    logger.debug('Updating your react-app-env.d.ts for modular-scripts');

    if (fs.existsSync(path.join(newPackagePath, 'src', 'react-app-env.d.ts'))) {
      fs.writeFileSync(path.join(newPackagePath, 'src', 'react-app-env.d.ts'), fs.readFileSync(path.join(newPackagePath, 'src', 'react-app-env.d.ts'), 'utf8').replace('<reference types="react-scripts" />', '<reference types="modular-scripts/react-app-env" />'));
    }

    fs.writeJSONSync(path.join(newPackagePath, 'tsconfig.json'), {
      extends: path.relative(newPackagePath, modularRoot) + '/tsconfig.json'
    }, {
      spaces: 2
    });
    logger.debug('Migrating setupTests file if it does not already exist');
    const setupFilesExts = ['setupTests.js', 'setupTests.ts'];
    const originSetupTest = setupFilesExts.find(file => {
      return fs.existsSync(path.join(newPackagePath, 'src', file));
    });
    const modularSetUpTest = setupFilesExts.find(file => {
      return fs.existsSync(path.join(modularRoot, 'modular', file));
    });

    if (originSetupTest && !modularSetUpTest) {
      fs.mkdirpSync(path.join(modularRoot, 'modular'));
      fs.writeFileSync(path.join(modularRoot, 'modular', originSetupTest), fs.readFileSync(path.join(newPackagePath, 'src', originSetupTest), 'utf8'));

      _rimraf.default.sync(path.join(newPackagePath, 'src', originSetupTest));
    }

    if (originSetupTest && modularSetUpTest) {
      logger.log(_chalk.default.gray('There is already a setupTests file present in the modular folder. ' + "Skipping porting the app's setupTests file over..."));

      _rimraf.default.sync(path.join(newPackagePath, 'src', originSetupTest));
    } // Staging ported package.json


    const {
      name,
      version,
      browserslist,
      private: prv
    } = targetedAppPackageJson;
    const stagedPackageJson = {
      name,
      version,
      browserslist,
      modular: {
        type: 'app'
      },
      private: prv
    };
    /* ****** NOTE ******
     * This does not set up 'nohoist' or 'exceptions' in workspaces
     * for mismatched versions. If the targeted app has a
     * dependency that is versioned differently than the modular root
     * dependency, the package & version in modular root will take precedence.
     *
     * If the targeted app has a devDependency that is marked
     * as a dependency in modular root, it will not be ported over
     * into the modular app as a devDependency but instead be kept as a
     * dependency in modular root.
     *
     * During this resolution, if modular root has the package in its
     * dependencies, the version in modular root will take precedence.
     *
     * This may affect the yarn workspaces mismatchedWorkspaceDependencies
     * property in workspace info.
     * (https://github.com/yarnpkg/yarn/issues/6898#issuecomment-478188695)
     *
     * Given the case that the app you are porting over has a dependency that is a
     * local package in the workspace, if the target app's dep has a different
     * version than the local version, that package would not be symlinked to
     * the local package at all. It would get its own copy in its node_modules.
     *
     * Example:
     * TargetApp's dependency: foo@^1.0.5
     * Modular package foo's local version: 2.0.1
     *
     * TargetApp will have copy of foo@1.0.5 in its workspace node_modules.
     * It will be marked as a mismatchedWorkspaceDependencies in yarn workspaces.
     *
     * We do not allow mismatchedWorkspaceDependencies in the modular workspace.
     * Thus, we will remove that dependency from the target app and have it use
     * the local symlinked version instead.
     *
     */

    logger.log('Resolving dependencies...');
    const modularRootPackageJson = await fs.readJSON(path.join(modularRoot, 'package.json'));
    const {
      dependencies: rootDeps = {},
      devDependencies: rootDevDeps = {}
    } = modularRootPackageJson;
    const {
      dependencies: targetDeps = {},
      devDependencies: targetDevDeps = {}
    } = targetedAppPackageJson;
    const workspaces = await (0, _getWorkspaceInfo.default)(modularRoot);
    const publicPackages = Object.keys(workspaces).filter(name => workspaces[name].public); // [targeted dep name] : root dep version that's taken precedence

    const droppedDeps = {}; // [targeted dep name] : local package version that's taken precedence

    const droppedWorkspaceDeps = {};
    const workspaceDeps = Object.keys(targetDeps || {}).reduce((acc, dep) => {
      // Do not bring over dep if it's react-scripts or if modular root has it
      // or if it is a public package in the modular workspace
      if (dep !== 'react-scripts') {
        if (!rootDeps[dep] && !publicPackages.includes(dep)) {
          return { ...acc,
            [dep]: targetDeps[dep]
          };
        } // if local package is marked as dependency, note it for later warnings


        if (publicPackages.includes(dep)) {
          droppedWorkspaceDeps[dep] = fs.readJsonSync(path.join(modularRoot, workspaces[dep].location, 'package.json')).version;
          return acc;
        } // if it modular workspace dep doesn't satisfy semver, note it for later warnings


        if (rootDeps[dep] && !_semver.default.satisfies(rootDeps[dep], targetDeps[dep])) {
          droppedDeps[dep] = rootDeps[dep];
          return acc;
        }
      }

      return acc;
    }, {});
    const workspaceDevDeps = Object.keys(targetDevDeps || {}).reduce((acc, devDep) => {
      // Do not bring over dep if it's react-scripts or if modular root has it
      // or if it is a public package in the modular workspace
      if (devDep !== 'react-scripts') {
        if (!rootDevDeps[devDep] && !rootDeps[devDep] && !publicPackages.includes(devDep)) {
          return { ...acc,
            [devDep]: targetDevDeps[devDep]
          };
        } // if local package is marked as dependency, note it for later warnings


        if (publicPackages.includes(devDep)) {
          droppedWorkspaceDeps[devDep] = fs.readJsonSync(path.join(modularRoot, workspaces[devDep].location, 'package.json')).version;
          return acc;
        } // if it modular workspace dep doesn't satisfy semver, note it for later warnings


        if (rootDeps[devDep] && !_semver.default.satisfies(rootDeps[devDep], targetDevDeps[devDep])) {
          droppedDeps[devDep] = rootDeps[devDep];
          return acc;
        }

        if (rootDevDeps[devDep] && !_semver.default.satisfies(rootDevDeps[devDep], targetDevDeps[devDep])) {
          droppedDeps[devDep] = rootDevDeps[devDep];
          return acc;
        }
      }

      return acc;
    }, {}); // Add updated dependencies to new app's package.json

    fs.writeJsonSync(path.join(newPackagePath, 'package.json'), { ...stagedPackageJson,
      dependencies: workspaceDeps,
      devDependencies: workspaceDevDeps
    }, {
      spaces: 2
    });
    logger.log('Installing dependencies...');

    _execa.default.sync('yarnpkg', ['--silent'], {
      cwd: modularRoot
    });

    logger.log('Validating your modular project...');
    await (0, _check.check)({
      fix: false,
      target: modularRoot
    });
    logger.log('Successfully ported your app over!');
    const droppedDepKeys = Object.keys(droppedDeps);
    const droppedWorkspaceKeys = Object.keys(droppedWorkspaceDeps);

    if (droppedDepKeys.length) {
      logger.warn('We found mismatched dependency versions between your targeted app and the root modular workspace');
      logger.warn('As part of the dependency resolution process, the dependencies in modular workspace were given precedence.');
      logger.warn(`Your targeted app (${targetedAppName}) will use these modular workspace dependencies:\n\n` + droppedDepKeys.map(dep => _chalk.default.bold(`  \u2022 ${dep}: ${droppedDeps[dep]}`)).join('\n') + '\n\n');
    }

    if (droppedWorkspaceKeys.length) {
      logger.warn('Your targeted app has a modular workspace public package marked as a dependency.');
      logger.warn("As part of the dependency resolution process, the local public package's version was given precedence.");
      logger.warn(`Your targeted app (${targetedAppName}) will use the local package version:\n\n` + droppedWorkspaceKeys.map(dep => _chalk.default.bold(`  \u2022 ${dep}: ${droppedWorkspaceDeps[dep]}`)).join('\n') + '\n\n');
    }

    if (droppedDepKeys.length || droppedWorkspaceDeps.length) {
      logger.error('You should look at the dependency resolution warning(s) above and confirm the current workspace dependencies will work for your app');
      logger.error('You may encounter issues later if you ignore these warnings.');
    }
  } catch (err) {
    logger.error(err);
    (0, _gitActions.stashChanges)();
  }
}

var _default = (0, _actionPreflightCheck.default)(port);

exports.default = _default;
//# sourceMappingURL=port.js.map